using System;
using NUnit.Framework;
using System.Linq;

namespace SharpSword.IntergrationTests
{
   [TestFixture]
   public class BibleModuleTests : TestFixtureBase
   {
      const string John3v16PlainTextExpected = "“For God so loved the world, that he gave his only Son, that whoever believes in him should not perish but have eternal life.";
      const string Numbers14v20PlainTextExpected = "Then the Lord said, “I have pardoned, according to your word.";


      [Test]
      public void GetVerse_Numbers14v20_CorrectText()
      {
         //Arrange
         var verseId = _bibleModule.Versification.CreateVerseIdentifier("Numbers", 14, 20);

         //Act
         var verse = _bibleModule.GetVerse(verseId);

         //Assert
         Assert.That (verse.Text, Is.EqualTo(Numbers14v20PlainTextExpected));
      }

      [Test]
      public void GetVerse_John3v16_CorrectText()
      {
         //Arrange
         var verseId = _bibleModule.Versification.CreateVerseIdentifier("John", 3, 16);

         //Act
         var verse = _bibleModule.GetVerse(verseId);

         //Assert
         Assert.That (verse.Text, Is.EqualTo(John3v16PlainTextExpected));
      }

      [Test]
      public void GetChapter_John3Plain_V16Correct()
      {
         //Arrange
         var verseId = _bibleModule.Versification.CreateVerseIdentifier("John", 3, 1);

         //Act
         var chapter = _bibleModule.GetChapter(verseId);

         //Assert
         Assert.That (chapter.ToArray()[15].Text, Is.EqualTo(John3v16PlainTextExpected));
      }
   }
}

