using System;
using NUnit.Framework;
using System.Linq;
using SharpSword.Formatters;

namespace SharpSword.IntergrationTests
{
   [TestFixture]
   public class LuceneIndexTests : TestFixtureBase
   {
      [Test]
      public void Search_God_FirstPageResults()
      {
         //Act
         var results =_bibleModule.Search("god", 0, 10).ToArray ();

         //Assert
         AssertVerseId(results[0], "Genesis", 1, 1);
         AssertVerseId(results[1], "Genesis", 1, 2);
         AssertVerseId(results[2], "Genesis", 1, 3);
         AssertVerseId(results[3], "Genesis", 1, 4);
         AssertVerseId(results[4], "Genesis", 1, 5);
         AssertVerseId(results[5], "Genesis", 1, 6);
         AssertVerseId(results[6], "Genesis", 1, 7);
         AssertVerseId(results[7], "Genesis", 1, 8);
         AssertVerseId(results[8], "Genesis", 1, 9);
         AssertVerseId(results[9], "Genesis", 1, 10);
      }

      [Test]
      public void Search_God_SecondPageResults()
      {
         //Act
         var results =_bibleModule.Search("god", 10, 10).ToArray ();

         //Assert
         AssertVerseId(results[0], "Genesis", 1, 11);
         AssertVerseId(results[1], "Genesis", 1, 12);
         AssertVerseId(results[2], "Genesis", 1, 14);
         AssertVerseId(results[3], "Genesis", 1, 16);
         AssertVerseId(results[4], "Genesis", 1, 17);
         AssertVerseId(results[5], "Genesis", 1, 18);
         AssertVerseId(results[6], "Genesis", 1, 20);
         AssertVerseId(results[7], "Genesis", 1, 21);
         AssertVerseId(results[8], "Genesis", 1, 22);
         AssertVerseId(results[9], "Genesis", 1, 24);
      }

      [Test]
      public void Search_God_LastPageResults()
      {
         //Act
         var results =_bibleModule.Search("god", 3861, 10).ToArray ();

         //Assert
         Assert.That (results.Length, Is.EqualTo (6));
         AssertVerseId(results[0], "Revelation", 22, 3);
         AssertVerseId(results[1], "Revelation", 22, 5);
         AssertVerseId(results[2], "Revelation", 22, 6);
         AssertVerseId(results[3], "Revelation", 22, 9);
         AssertVerseId(results[4], "Revelation", 22, 18);
         AssertVerseId(results[5], "Revelation", 22, 19);
      }

      [Test]
      public void Search_MultiWordSearch_LastPageResults()
      {
         //Act
         var results =_bibleModule.Search("god so loved the world", 0, 10).ToArray ();

         //Assert
         Assert.That (results.Length, Is.EqualTo (1));
         AssertVerseId(results[0], "John", 3, 16);
      }

      [Test]
      public void Search_BookFilter_OnlyResultsFromThatBook()
      {
         //Act
         var results =_bibleModule.Search("god", "Job", 0, 10).ToArray ();

         //Assert
         Assert.That (results.Length, Is.EqualTo(10));
         Assert.That (results.All(verse => verse.BookName == "Job"));
      }

      [TestCase("Genesis", 199)]
      [TestCase("Exodus", 102)]
      [TestCase("Revelation", 87)]
      public void GetBookFrequencies_God_CorrectFrequencies(string book, int expectedCount)
      {
         //Act
         var frequencies = _bibleModule.GetBookFrequencies("god");
         
         //Assert
         Assert.That (frequencies[book], Is.EqualTo(expectedCount));
      }

      [Test]
      public void GetBookFrequencies_God_DoesntIncludeBooksWithNoHits()
      {
         //Act
         var frequencies = _bibleModule.GetBookFrequencies("god");

         //Assert
         Assert.That (frequencies.ContainsKey("Esther"), Is.False);
      }

      private void AssertVerseId(IVerseIdentifier verseIdentifier, string book, int chapter, int verse)
      {
         Assert.That (verseIdentifier.BookName, Is.EqualTo(book));
         Assert.That (verseIdentifier.ChapterNumber, Is.EqualTo(chapter));
         Assert.That (verseIdentifier.VerseNumber, Is.EqualTo(verse));
      }
   }
}

