using System;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework;
using System.IO;

namespace SharpSword.IntergrationTests
{
   [TestFixture]
   public class ModuleManagerTests
   {
      ModuleManager _moduleManager;

      [TestFixtureSetUp]
      public void FixtureSetup()
      {
         _moduleManager = ModuleManager.Instance;
      }

      [Test]
      public void GetDownloadableModules_ContainsExpectedModules ()
      {
         //Act
         var modules = _moduleManager.GetDownloadableModules();

         //Assert
         Assert.That (modules.Where(module => module.Name == "KJV").Any(), Is.True);
         Assert.That (modules.Where(module => module.Name == "ESV").Any(), Is.True);
      }

      [Test]
      public void InstallModule_Installed()
      {
         //Arrange
         var configFile = Path.Combine(SetupTests.SWORD_MODD, "esv.conf");
         if(File.Exists (configFile))
         {
            File.Delete (configFile);
         }
         var moduleFolder = Path.Combine (SetupTests.SWORD_APPDATA, "modules", "texts", "ztext", "esv");
         if(Directory.Exists(moduleFolder))
         {
            Directory.Delete(moduleFolder, true);
         }

         //Act
         _moduleManager.InstallModule("esv");

         //Assert
         Assert.That (File.Exists (configFile), Is.True);
         Assert.That (Directory.Exists (moduleFolder), Is.True);
         Assert.That (File.Exists (Path.Combine(moduleFolder,"nt.bzs")), Is.True);
         Assert.That (File.Exists (Path.Combine(moduleFolder,"nt.bzv")), Is.True);
         Assert.That (File.Exists (Path.Combine(moduleFolder,"nt.bzz")), Is.True);
         Assert.That (File.Exists (Path.Combine(moduleFolder,"ot.bzs")), Is.True);
         Assert.That (File.Exists (Path.Combine(moduleFolder,"ot.bzv")), Is.True);
         Assert.That (File.Exists (Path.Combine(moduleFolder,"ot.bzz")), Is.True);
         Assert.That (Directory.Exists (Path.Combine(moduleFolder,"lucenenet")), Is.True);
      }
   }
}

