using System;
using NUnit.Framework;

namespace SharpSword.Tests
{
   [TestFixture]
   public class OsisFormatterTests
   {
      [Test]
      public void OsisFormatter_FormatGen1v1_ReturnsOriginal()
      {
         //arrange
         var osisText = "<title subType=\"x-preverse\" type=\"section\">The Beginning of Creation</title>In the <note n=\"a\" osisID=\"Gen.1.1!crossReference.a\" osisRef=\"Gen.1.1\" type=\"crossReference\"><reference osisRef=\"Job.38.4-Job.38.7\">Job 38:4-7</reference>; <reference osisRef=\"Ps.33.6\">Ps. 33:6</reference>; <reference osisRef=\"Ps.136.5\">136:5</reference>; <reference osisRef=\"Isa.42.5\">Isa. 42:5</reference>; <reference osisRef=\"Isa.45.18\">45:18</reference>; <reference osisRef=\"John.1.1-John.1.3\">John 1:1-3</reference>; <reference osisRef=\"Acts.14.15\">Acts 14:15</reference>; <reference osisRef=\"Acts.17.24\">17:24</reference>; <reference osisRef=\"Col.1.16-Col.1.17\">Col. 1:16, 17</reference>; <reference osisRef=\"Heb.1.10\">Heb. 1:10</reference>; <reference osisRef=\"Heb.11.3\">11:3</reference>; <reference osisRef=\"Rev.4.11\">Rev. 4:11</reference></note>beginning, God created the heavens and the earth.";
         var formatter = new OsisFormatter();

         //Act
         var formattedText = formatter.Format(osisText, new VerseIdentifier("Genesis",1,1));

         //Assert
         Assert.That (formattedText, Is.EqualTo (osisText));
      }
      
      [Test]
      public void ChapterTitle_Genesis1_ReturnsEmptyString()
      {
         //arrange
         var formatter = new OsisFormatter();
         var verseIdentifier = new VerseIdentifier("Genesis",1,1);

         //Act
         var formattedText = formatter.ChapterTitle(verseIdentifier);

         //Assert
         Assert.That (formattedText, Is.EqualTo (string.Empty));
      }
   }
}

