using System;
using NUnit.Framework;
using SharpSword;
using System.Collections.Generic;
using System.Linq;

namespace SharpSword.Tests
{
   [TestFixture]
   public class KjvVersificationTests
   {
      [Test]
      public void NextChapter_Gen1_Gen2 ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         var startingVerse = versification.CreateVerseIdentifier ("Genesis", 1, 1);
         var expectedNextVerse = versification.CreateVerseIdentifier ("Genesis", 2, 1);
       
         //act
         var nextVerse = versification.NextChapter (startingVerse);
          
         //assert
         Assert.That (nextVerse, Is.EqualTo (expectedNextVerse));
      }
    
      [Test]
      public void NextChapter_Gen50_Exo1 ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         var startingVerse = versification.CreateVerseIdentifier ("Genesis", 50, 1);
         var expectedNextVerse = versification.CreateVerseIdentifier ("Exodus", 1, 1);
       
         //act
         var nextVerse = versification.NextChapter (startingVerse);
          
         //assert
         Assert.That (nextVerse, Is.EqualTo (expectedNextVerse));
      }
    
      [Test]
      public void NextChapter_Mal4_Matt1 ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         var startingVerse = versification.CreateVerseIdentifier ("Malachi", 4, 1);
         var expectedNextVerse = versification.CreateVerseIdentifier ("Matthew", 1, 1);
       
         //act
         var nextVerse = versification.NextChapter (startingVerse);
          
         //assert
         Assert.That (nextVerse, Is.EqualTo (expectedNextVerse));
      }
    
      [Test]
      public void NextChapter_Rev21_Null ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         var startingVerse = versification.CreateVerseIdentifier ("Revelation", 21, 1);
       
         //act
         var nextVerse = versification.NextChapter (startingVerse);
          
         //assert
         Assert.That (nextVerse, Is.Null);
      }
    
      [Test]
      public void PreviousChapter_Rev21_Rev20 ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         var startingVerse = versification.CreateVerseIdentifier ("Revelation", 21, 1);
         var expectedPreviousVerse = versification.CreateVerseIdentifier ("Revelation", 20, 1);
       
         //act
         var previousVerse = versification.PreviousChapter (startingVerse);
          
         //assert
         Assert.That (previousVerse, Is.EqualTo (expectedPreviousVerse));
      }
    
      [Test]
      public void PreviousChapter_Rev1_Jud1 ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         var startingVerse = versification.CreateVerseIdentifier ("Revelation", 1, 1);
         var expectedPreviousVerse = versification.CreateVerseIdentifier ("Jude", 1, 1);
       
         //act
         var previousVerse = versification.PreviousChapter (startingVerse);
          
         //assert
         Assert.That (previousVerse, Is.EqualTo (expectedPreviousVerse));
      }
    
      [Test]
      public void PreviousChapter_Matt1_Mal4 ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         var startingVerse = versification.CreateVerseIdentifier ("Matthew", 1, 1);
         var expectedPreviousVerse = versification.CreateVerseIdentifier ("Malachi", 4, 1);
       
         //act
         var previousVerse = versification.PreviousChapter (startingVerse);
          
         //assert
         Assert.That (previousVerse, Is.EqualTo (expectedPreviousVerse));
      }
    
      [Test]
      public void PreviousChapter_Gen1_Null ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         var startingVerse = versification.CreateVerseIdentifier ("Genesis", 1, 1);
       
         //act
         var previousVerse = versification.PreviousChapter (startingVerse);
          
         //assert
         Assert.That (previousVerse, Is.Null);
      }
    
      [Test]
      public void GetBookNames_Returns66Books ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
       
         //act
         var bookNames = versification.GetBookNames ();
       
         //assert
         Assert.That (bookNames.Count (), Is.EqualTo (66));
      }
    
      [Test]
      public void GetBookNames_ReturnsCorrectBooks ()
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
       
         string[] expectedBooks = new string[]
         {
            "Genesis", "Exodus", "Leviticus", "Numbers",
            "Deuteronomy", "Joshua", "Judges", "Ruth",
            "1 Samuel", "2 Samuel", "1 Kings", "2 Kings",
            "1 Chronicles", "2 Chronicles", "Ezra", "Nehemiah",
            "Esther", "Job", "Psalms", "Proverbs", "Ecclesiastes",
            "Song of Solomon", "Isaiah", "Jeremiah", "Lamentations",
            "Ezekiel", "Daniel", "Hosea", "Joel",
            "Amos", "Obadiah", "Jonah", "Micah",
            "Nahum", "Habakkuk", "Zephaniah", "Haggai",
            "Zechariah", "Malachi", "Matthew", "Mark",
            "Luke", "John", "Acts", "Romans",
            "1 Corinthians", "2 Corinthians", "Galatians", "Ephesians",
            "Philippians", "Colossians", "1 Thessalonians", "2 Thessalonians",
            "1 Timothy", "2 Timothy", "Titus", "Philemon",
            "Hebrews", "James", "1 Peter", "2 Peter",
            "1 John",  "2 John", "3 John", "Jude",
            "Revelation"
         };

         //act
         var bookNames = versification.GetBookNames ();

         //assert
         Assert.That (bookNames.SequenceEqual (expectedBooks));
      }

      [TestCase("Psalms", 150, 6)]
      [TestCase("Revelation", 22, 21)]
      [TestCase("Genesis", 1, 1)]
      public void IsValidVerse_Valid_ReturnsTrue(string book, int chapter, int verse)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //act
         var result = versification.IsValidVerse(book, chapter, verse);

         //assert
         Assert.That (result, Is.True);
      }

      [TestCase("Psalmss", 150, 6)]
      [TestCase("Psalms", 151, 6)]
      [TestCase("Psalms", 150, 7)]
      public void IsValidVerse_Invalid_ReturnsFalse(string book, int chapter, int verse)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //act
         var result = versification.IsValidVerse(book, chapter, verse);

         //assert
         Assert.That (result, Is.False);
      }

      [TestCase("Psalms", 150, 6)]
      [TestCase("Revelation", 22, 21)]
      [TestCase("Genesis", 1, 1)]
      public void TryCreateVerseIdentifier_Valid_ReturnsTrue(string book, int chapter, int verse)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //act
         var result = versification.TryCreateVerseIdentifier(book, chapter, verse, out verseIdentifier);

         //assert
         Assert.That (result, Is.True);
      }

      [TestCase("Psalms", 150, 6)]
      [TestCase("Revelation", 22, 21)]
      [TestCase("Genesis", 1, 1)]
      public void TryCreateVerseIdentifier_Valid_OutCorrectVerse(string book, int chapter, int verse)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //act
         versification.TryCreateVerseIdentifier(book, chapter, verse, out verseIdentifier);

         //assert
         Assert.That (verseIdentifier.BookName, Is.EqualTo(book));
         Assert.That (verseIdentifier.ChapterNumber, Is.EqualTo(chapter));
         Assert.That (verseIdentifier.VerseNumber, Is.EqualTo(verse));
      }

      [TestCase("Psalmss", 150, 6)]
      [TestCase("Psalms", 151, 6)]
      [TestCase("Psalms", 150, 7)]
      public void TryCreateVerseIdentifier_Invalid_ReturnsFalse(string book, int chapter, int verse)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //act
         var result = versification.TryCreateVerseIdentifier(book, chapter, verse, out verseIdentifier);

         //assert
         Assert.That (result, Is.False);
      }

      [TestCase("Psalms", 150, 6)]
      [TestCase("Revelation", 22, 21)]
      [TestCase("Genesis", 1, 1)]
      public void CreateVerseIdentifier_Valid_ReturnsVerseIdentifier(string book, int chapter, int verse)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //act
         var verseIdentifier = versification.CreateVerseIdentifier(book, chapter, verse);

         //assert
         Assert.That (verseIdentifier.BookName, Is.EqualTo(book));
         Assert.That (verseIdentifier.ChapterNumber, Is.EqualTo(chapter));
         Assert.That (verseIdentifier.VerseNumber, Is.EqualTo(verse));
      }

      [TestCase("Psalmss", 150, 6)]
      [TestCase("Psalms", 151, 6)]
      [TestCase("Psalms", 150, 7)]
      [ExpectedException(typeof(ArgumentOutOfRangeException))]
      public void CreateVerseIdentifier_InvalidVerse_ThrowsException(string book, int chapter, int verse)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //act
         versification.CreateVerseIdentifier(book, chapter, verse);
      }

      [TestCase("1 John", 5)]
      [TestCase("Esther", 10)]
      public void GetChapters_1John_CorrectNumberOfChapters(string book, int numberOfChapters)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //act
         var chapters = versification.GetChapters(book);

         //assert
         Assert.That (chapters.Count (), Is.EqualTo(numberOfChapters));
      }

      [TestCase("1 John")]
      [TestCase("Esther")]
      public void GetChapters_ValidBook_ChaptersInSequence(string book)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //act
         var chapters = versification.GetChapters(book);

         //assert
         int chapterNumber = 1;
         foreach(var chapter in chapters)
         {
            Assert.That (chapter.ChapterNumber, Is.EqualTo(chapterNumber++));
         }
      }

      [TestCase("1 John")]
      [TestCase("Esther")]
      public void GetChapters_ValidBook_ChaptersInCorrectBook(string book)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //act
         var chapters = versification.GetChapters(book);

         //assert
         foreach(var chapter in chapters)
         {
            Assert.That (chapter.BookName, Is.EqualTo(book));
         }
      }

      [TestCase("1 John")]
      [TestCase("Esther")]
      public void GetChapters_ValidBook_VersesOne(string book)
      {
         //arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //act
         var chapters = versification.GetChapters(book);

         //assert
         foreach(var chapter in chapters)
         {
            Assert.That (chapter.VerseNumber, Is.EqualTo(1));
         }
      }

      [Test]
      public void TryCreateVerseIdentifier_ValidVerse_ReturnsTrue ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool valid =  versification.TryCreateVerseIdentifier("Job 2:4", out verseIdentifier);

         //Assert
         Assert.That (valid, Is.True);
      }
      
      [Test]
      public void TryCreateVerseIdentifier_JustValidBook_ReturnsTrue ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool valid =  versification.TryCreateVerseIdentifier("Genesis", out verseIdentifier);

         //Assert
         Assert.That (valid, Is.True);
      }
      
      [Test]
      public void TryCreateVerseIdentifier_JustInvalidBook_ReturnsFalse ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool valid =  versification.TryCreateVerseIdentifier("Isers", out verseIdentifier);

         //Assert
         Assert.That (valid, Is.False);
      }
      
      [Test]
      public void TryCreateVerseIdentifier_JustValidBookAndChapter_ReturnsTrue ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool valid =  versification.TryCreateVerseIdentifier("Revelation 22", out verseIdentifier);

         //Assert
         Assert.That (valid, Is.True);
      }
      
      [Test]
      public void TryCreateVerseIdentifier_BookWithSpace_ReturnsTrue ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool valid =  versification.TryCreateVerseIdentifier("Revelation ", out verseIdentifier);

         //Assert
         Assert.That (valid, Is.True);
      }
      
      [Test]
      public void TryCreateVerseIdentifier_BookWithChapterEndsWithColon_ReturnsTrue ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool valid =  versification.TryCreateVerseIdentifier("Revelation 12:", out verseIdentifier);

         //Assert
         Assert.That (valid, Is.True);
      }
      
      [Test]
      public void TryCreateVerseIdentifier_BookValidAndChapterToHighNoVerse_ReturnsFalse ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool valid =  versification.TryCreateVerseIdentifier("Revelation 23", out verseIdentifier);

         //Assert
         Assert.That (valid, Is.False);
      }

      [Test]
      public void TryCreateVerseIdentifier_ValidVerse_CorrectVerseReturned ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         versification.TryCreateVerseIdentifier("Job 2:4", out verseIdentifier);

         //Assert
         Assert.That (verseIdentifier, Is.EqualTo(new VerseIdentifier("Job", 2, 4)));
      }

      [Test]
      public void CreateVerseIdentifier_ValidVerse_CorrectVerseReturned ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //Act
         var verseIdentifier = versification.CreateVerseIdentifier("Job 2:4");

         //Assert
         Assert.That (verseIdentifier, Is.EqualTo(new VerseIdentifier("Job", 2, 4)));
      }

      [Test]
      public void TryCreateVerseIdentifier_MissingSpace_ReturnsFalse ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool result = versification.TryCreateVerseIdentifier("Job2:4", out verseIdentifier);

         //Assert
         Assert.That (result, Is.False);
      }

      [Test]
      [ExpectedException(typeof(ArgumentOutOfRangeException))]
      public void CreateVerseIdentifier_MissingSpace_ThrowException()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");

         //Act
         versification.CreateVerseIdentifier("Job2:4");
      }
      
      [Test]
      public void TryCreateVerseIdentifier_InSearch_ReturnsFalse ()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool result = versification.TryCreateVerseIdentifier("separate in:Romans", out verseIdentifier);

         //Assert
         Assert.That (result, Is.False);
      }
      
      [TestCase(" A")]
      [TestCase(" :0")]
      [TestCase("0: 0")]
      public void TryCreateVerseIdentifier_NotAVerse_ReturnsFalse (string notAverse)
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool result = versification.TryCreateVerseIdentifier(notAverse, out verseIdentifier);

         //Assert
         Assert.That (result, Is.False);
      }
      
      [Test]
      public void TryCreateVerseIdentifier_TwoPartBookNoVerse_CreatesCorrectVerse()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool result = versification.TryCreateVerseIdentifier("2 Samuel 10", out verseIdentifier);

         //Assert
         Assert.That (result, Is.True);
         Assert.That (verseIdentifier.BookName, Is.EqualTo ("2 Samuel"));
         Assert.That (verseIdentifier.ChapterNumber, Is.EqualTo (10));
         Assert.That (verseIdentifier.VerseNumber, Is.EqualTo (1));
      }
      
      [Test]
      public void TryCreateVerseIdentifier_TwoPartBookNoChapter_CreatesCorrectVerse()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool result = versification.TryCreateVerseIdentifier("1 John", out verseIdentifier);

         //Assert
         Assert.That (result, Is.True);
         Assert.That (verseIdentifier.BookName, Is.EqualTo ("1 John"));
         Assert.That (verseIdentifier.ChapterNumber, Is.EqualTo (1));
         Assert.That (verseIdentifier.VerseNumber, Is.EqualTo (1));
      }
      
      [Test]
      public void TryCreateVerseIdentifier_BookOnlyWithSmalls_CreatesCorrectVerse()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool result = versification.TryCreateVerseIdentifier("john", out verseIdentifier);

         //Assert
         Assert.That (result, Is.True);
         Assert.That (verseIdentifier.BookName, Is.EqualTo ("John"));
         Assert.That (verseIdentifier.ChapterNumber, Is.EqualTo (1));
         Assert.That (verseIdentifier.VerseNumber, Is.EqualTo (1));
      }
      
      [Test]
      public void TryCreateVerseIdentifier_BookAndChapterWithSmalls_CreatesCorrectVerse()
      {
         //Arrange
         IVersification versification = VersificationManager.Instance.GetVersification ("kjv");
         IVerseIdentifier verseIdentifier;

         //Act
         bool result = versification.TryCreateVerseIdentifier("psalms 3", out verseIdentifier);

         //Assert
         Assert.That (result, Is.True);
         Assert.That (verseIdentifier.BookName, Is.EqualTo ("Psalms"));
         Assert.That (verseIdentifier.ChapterNumber, Is.EqualTo (3));
         Assert.That (verseIdentifier.VerseNumber, Is.EqualTo (1));
      }
   }
}

