using System;
using NUnit.Framework;
using SharpSword;
using System.Collections.Generic;
using System.Linq;

namespace SharpSword.Tests
{
   [TestFixture]
   public class VerseIdentifierTests
   {
      [Test]
      public void Bookname_Genesis_ReturnedFromProperty()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier = versification.CreateVerseIdentifier("Genesis", 1,1);

         //Assert
         Assert.That(verseIdentifier.BookName, Is.EqualTo ("Genesis"));
      }

      [Test]
      public void ChapterNumber_42_SameAsProperty()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier = versification.CreateVerseIdentifier("Genesis", 42,1);

         //Assert
         Assert.That(verseIdentifier.ChapterNumber, Is.EqualTo (42));
      }

      [Test]
      public void VerseNumber_23_SameAsProperty()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier = versification.CreateVerseIdentifier("Genesis", 1,23);

         //Assert
         Assert.That(verseIdentifier.VerseNumber, Is.EqualTo (23));
      }

      [Test]
      public void EqualsObject_SameVerseDifferentObject_ReturnsTrue()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         object verseIdentifier2 = versification.CreateVerseIdentifier("Job", 3, 4);

         //Assert
         Assert.That(verseIdentifier1.Equals(verseIdentifier2), Is.True);
      }

      [Test]
      public void EqualsObject_SameObject_ReturnsTrue()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         object verseIdentifier = versification.CreateVerseIdentifier("Job", 3, 4);

         //Assert
         Assert.That(verseIdentifier.Equals(verseIdentifier), Is.True);
      }

      [Test]
      public void EqualsObject_Null_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         object verseIdentifier = versification.CreateVerseIdentifier("Job", 3, 4);

         //Assert
         Assert.That(verseIdentifier.Equals(null), Is.False);
      }

      [Test]
      public void EqualsObject_DifferentBook_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         object verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         object verseIdentifier2 = versification.CreateVerseIdentifier("Psalms", 3, 4);


         //Assert
         Assert.That(verseIdentifier1.Equals(verseIdentifier2), Is.False);
      }

      [Test]
      public void EqualsObject_DifferentChapter_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         object verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         object verseIdentifier2 = versification.CreateVerseIdentifier("Job", 4, 4);


         //Assert
         Assert.That(verseIdentifier1.Equals(verseIdentifier2), Is.False);
      }

      [Test]
      public void EqualsObject_DifferentVerse_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         object verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         object verseIdentifier2 = versification.CreateVerseIdentifier("Job", 3, 5);


         //Assert
         Assert.That(verseIdentifier1.Equals(verseIdentifier2), Is.False);
      }

      [Test]
      public void Equals_SameVerseDifferentObject_ReturnsTrue()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         var verseIdentifier2 = versification.CreateVerseIdentifier("Job", 3, 4);

         //Assert
         Assert.That(verseIdentifier1.Equals(verseIdentifier2), Is.True);
      }

      [Test]
      public void Equals_SameObject_ReturnsTrue()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier = versification.CreateVerseIdentifier("Job", 3, 4);

         //Assert
         Assert.That(verseIdentifier.Equals(verseIdentifier), Is.True);
      }

      [Test]
      public void Equals_Null_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier = versification.CreateVerseIdentifier("Job", 3, 4);
         VerseIdentifier nullVerse = null;

         //Assert
         Assert.That(verseIdentifier.Equals(nullVerse), Is.False);
      }

      [Test]
      public void Equals_DifferentBook_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         var verseIdentifier2 = versification.CreateVerseIdentifier("Psalms", 3, 4);


         //Assert
         Assert.That(verseIdentifier1.Equals(verseIdentifier2), Is.False);
      }

      [Test]
      public void Equals_DifferentChapter_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         var verseIdentifier2 = versification.CreateVerseIdentifier("Job", 4, 4);


         //Assert
         Assert.That(verseIdentifier1.Equals(verseIdentifier2), Is.False);
      }

      [Test]
      public void Equals_DifferentVerse_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");

         //Act
         var verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         var verseIdentifier2 = versification.CreateVerseIdentifier("Job", 3, 5);

         //Assert
         Assert.That(verseIdentifier1.Equals(verseIdentifier2), Is.False);
      }

      [Test]
      public void GetHashCode_SameObject_SameHashCode()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");
         var verseIdentifier = versification.CreateVerseIdentifier("Job", 3, 4);

         //Act
         int firstHash = verseIdentifier.GetHashCode();
         int secondHash = verseIdentifier.GetHashCode();

         //Assert
         Assert.That (firstHash, Is.EqualTo(secondHash));
      }

      [Test]
      public void GetHashCode_SameContents_SameHashCode()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");
         var verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         var verseIdentifier2 = versification.CreateVerseIdentifier("Job", 3, 4);

         //Act
         int firstHash = verseIdentifier1.GetHashCode();
         int secondHash = verseIdentifier2.GetHashCode();

         //Assert
         Assert.That (firstHash, Is.EqualTo(secondHash));
      }

      /// <summary>
      /// Different contents does not guarantee a different hash,
      /// But in most cases it should and in this case it does.
      /// </summary>
      [Test]
      public void GetHashCode_DifferentContents_DifferentHashCode()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");
         var verseIdentifier1 = versification.CreateVerseIdentifier("Job", 3, 4);
         var verseIdentifier2 = versification.CreateVerseIdentifier("Job", 3, 5);

         //Act
         int firstHash = verseIdentifier1.GetHashCode();
         int secondHash = verseIdentifier2.GetHashCode();

         //Assert
         Assert.That (firstHash, Is.Not.EqualTo(secondHash));
      }

      [Test]
      public void EqualsOperator_SameObject_ReturnsTrue()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");
         var verseIdentifier1 = versification.CreateVerseIdentifier("John", 3, 4);
         var verseIdentifier2 = verseIdentifier1;

         //Act
         bool result = verseIdentifier1 == verseIdentifier2;

         //Assert
         Assert.That (result, Is.True);
      }

      [Test]
      public void EqualsOperator_SameContents_ReturnsTrue()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");
         var verseIdentifier1 = versification.CreateVerseIdentifier("John", 3, 4);
         var verseIdentifier2 = versification.CreateVerseIdentifier("John", 3, 4);


         //Act
         bool result = verseIdentifier1.Equals(verseIdentifier2);

         //Assert
         Assert.That (result, Is.True);
      }

      [Test]
      public void EqualsOperator_Null_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");
         var verseIdentifier = versification.CreateVerseIdentifier("John", 3, 4);

         //Act
         bool result = verseIdentifier == null;

         //Assert
         Assert.That (result, Is.False);
      }

      [Test]
      public void EqualsOperator_DifferentContents_ReturnsFalse()
      {
         //Arrange
         var versification = VersificationManager.Instance.GetVersification("kjv");
         var verseIdentifier1 = versification.CreateVerseIdentifier("Matthew", 3, 4);
         var verseIdentifier2 = versification.CreateVerseIdentifier("John", 3, 4);

         //Act
         bool result = verseIdentifier1 == verseIdentifier2;

         //Assert
         Assert.That (result, Is.False);
      }
   }
}

