using System;
using System.Xml;
using System.Xml.Linq;
using System.Linq;
using System.Text;
using System.Collections.Generic;

namespace SharpSword.Formatters
{
   class PlainFormatter : IFormatter
   {
      public string Header()
      {
         return string.Empty;
      }
      
      public string Format (string text, IVerseIdentifier verseNumber)
      {
         text = text.Replace("\n",string.Empty);
         while(text.Contains("  "))
            text = text.Replace("  "," ");
         XElement verse = XElement.Parse("<v>" + text + "</v>",LoadOptions.PreserveWhitespace);
         var stringBuilder = new StringBuilder();
         ParseNode(verse, stringBuilder);
         return stringBuilder.ToString();
      }
      
      public string Footer()
      {
         return string.Empty;
      }

      void ParseNode(XElement element, StringBuilder stringBuilder)
      {
         foreach (XNode node in element.Nodes())
         {
            if(node.NodeType == XmlNodeType.Element)
            {
               var elementNode = node as XElement;
               if(elementNode.Name == "q")
               {
                  ProcessQuoteElement(elementNode, stringBuilder);
               }
               else if(elementNode.Name == "milestone")
               {
                  ProcessMilestoneElement(elementNode, stringBuilder);
               }
               else if(elementNode.Name == "l")
               {
                  ProcessLineElement(elementNode, stringBuilder);
               }
               else if(elementNode.Name == "divineName")
               {
                  ParseNode ((XElement)elementNode, stringBuilder);
               }
               else if(elementNode.Name == "lb")
               {
                  stringBuilder.Append(Environment.NewLine);
               }
            }
            else if(node.NodeType == XmlNodeType.Text)
            {
               stringBuilder.Append(node.ToString());
            }
         }
      }

      void ProcessQuoteElement(XElement elementNode, StringBuilder stringBuilder)
      {
         string quoteMarker = GetQuoteMarker(elementNode);

         if(!elementNode.Attributes("sID").Any() && !elementNode.Attributes("eID").Any())
         {
            stringBuilder.Append(quoteMarker);
         }

         ParseNode (elementNode, stringBuilder);
         stringBuilder.Append(quoteMarker);
      }

      void ProcessLineElement(XElement elementNode, StringBuilder stringBuilder)
      {
         ParseNode ((XElement)elementNode, stringBuilder);
         if(!elementNode.Attributes("sID").Any())
         {
            stringBuilder.Append(Environment.NewLine);
         }
      }

      void ProcessMilestoneElement(XElement elementNode, StringBuilder stringBuilder)
      {
         if(elementNode.Attributes("type").Any() && elementNode.Attribute("type").Value == "cQuote")
         {
            stringBuilder.Append(GetQuoteMarker(elementNode));
         }
      }

      string GetQuoteMarker(XElement quoteElement)
      {
         var markers = quoteElement.Attributes("marker");
         if(!markers.Any())
         {
            return "\"";
         }
         return (string)markers.First();
      }
      
      public bool IncludeVerseNumbers
      {
         get;
         set;
      }
      
      public string ChapterTitle (IVerseIdentifier verseIdentifier)
      {
         return string.Format ("Chapter {0}{1}", verseIdentifier.ChapterNumber, Environment.NewLine);
      }
   }
}

