// SharpSword C# library for accessing sword modules
// Copyright (C) 2013  Daniel Hughes
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using SharpSword.Formatters;

namespace SharpSword
{
   public interface IBibleModule : IDisposable
   {
      IEnumerable<IVerseIdentifier> Search (string searchTerm, int start, int numberOfHits);

      IEnumerable<IVerseIdentifier> Search (string searchTerm, string book, int start, int numberOfHits);

      IDictionary<string, int> GetBookFrequencies(string searchTerm);

      /// <summary>
      /// Gets the verse. Using the plain formatter.
      /// </summary>
      /// <returns>
      /// The verse.
      /// </returns>
      /// <param name='verseIdentifier'>
      /// Identifies the verse to get
      /// </param>
      IVerse GetVerse (IVerseIdentifier verseIdentifier);

      /// <summary>
      /// Gets the verse.
      /// </summary>
      /// <returns>
      /// The verse.
      /// </returns>
      /// <param name='verseIdentifier'>
      /// Identifies the verse to get
      /// </param>
      /// <param name='formatter'>
      /// The formatter to use for the verse text.
      /// </param>
      IVerse GetVerse (IVerseIdentifier verseIdentifier, IFormatter formatter);

      /// <summary>
      /// Gets the chapter using the plain formatter
      /// </summary>
      /// <returns>
      /// The verses making up a chapter.
      /// </returns>
      /// <param name='chapter'>
      /// A verse identifier for a verse in the chapter.
      /// </param>
      IEnumerable<IVerse> GetChapter (IVerseIdentifier chapter);

      /// <summary>
      /// Gets the chapter using the specified formatter
      /// </summary>
      /// <returns>
      /// The verses making up a chapter.
      /// </returns>
      /// <param name='chapter'>
      /// A verse identifier for a verse in the chapter.
      /// </param>
      /// <param name=formatter>formatter to use for the verse text.</param>
      IEnumerable<IVerse> GetChapter (IVerseIdentifier chapter, IFormatter formatter);
    
      IVersification Versification
      {
         get;   
      }
   }
}

