// WideMargin. Simple fast bible software.
// Copyright (C) 2013  Daniel Hughes
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.IO;
using System.Text;
using System.Linq;
using System.Collections.Generic;

namespace SharpSword
{
   public class ModuleConfig
      : IModuleConfig
   {
      private Dictionary<string,List<string>> _tags;

      public ModuleConfig (string file)
      {
         _tags = new Dictionary<string,List<string>> ();
         string[] lines = File.ReadAllLines (file);
         Name = lines [0].Replace ("[", string.Empty).Replace ("]", string.Empty);
         foreach (string line in lines.Skip(1))
         {
            if (line.Contains ("="))
            {
               string[] parts = line.Split ('=');
               string tag = parts [0];
               string val = parts [1];
               if (_tags.ContainsKey (tag))
               {
                  _tags [tag].Add (val);
               }
               else
               {
                  _tags.Add (tag, new List<string> (){val});
               }
            }
         }
      }
    
      public virtual string Name
      {
         get;
         private set;
      }
    
      public virtual bool IsBible
      {
         get
         {
            return GetEntry ("LCSH").Any (val => val.Contains ("Bible"));   
         }
      }
    
      public virtual bool Osis
      {
         get
         {
            return GetEntry ("SourceType").Any (val => val.Contains ("OSIS"));
         }
      }
    
      public virtual string DataPath
      {
         get
         {
            return GetEntry ("DataPath").FirstOrDefault ();  
         }
      }
    
      /// <summary>
      /// Gets the entry.
      /// </summary>
      /// <returns>
      /// The entry or string.Empty if it was not found
      /// </returns>
      /// <param name='entryName'>
      /// Entry name.
      /// </param>
      public virtual IEnumerable<string> GetEntry (string entryName)
      {
         List<string> entryValue = null;
         if (_tags.TryGetValue (entryName, out entryValue))
         {
            return entryValue;
         }
         return new string[]{};
      }
    
      public virtual string Versification
      {
         get
         {
            var versifications = GetEntry ("Versification");
            if (!versifications.Any ())
            {
               return "kjv";
            }
            return    versifications.First ();
         }
      }
   }
}

