// SharpSword C# library for accessing sword modules
// Copyright (C) 2013  Daniel Hughes
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
using System;

namespace SharpSword
{
   internal class VerseIdentifier : IVerseIdentifier
   {
      internal VerseIdentifier (string bookName, int chapterNumber, int verseNumber)
      {
         BookName = bookName;
         ChapterNumber = chapterNumber;
         VerseNumber = verseNumber;
      }

      public string BookName
      {
         get;
         private set;
      }
    
      public int ChapterNumber
      {
         get;
         private set;
      }
    
      public int VerseNumber
      {
         get;
         private set;
      }
    
      internal static VerseIdentifier ParseFromLuceneKey (string luceneKey, IVersificationInternal versification)
      {
         var keyParts = luceneKey.Split ('.');
         string bookName = versification.GetBookName (keyParts [0].ToLower ());
         return new VerseIdentifier (bookName, int.Parse (keyParts [1]), int.Parse (keyParts [2]));
      }
    
      public override string ToString ()
      {
         return string.Format ("{0} {1}:{2}", BookName, ChapterNumber, VerseNumber);
      }

      public override bool Equals (object obj)
      {
         if (obj == null)
         {
            return false;
         }
 
         VerseIdentifier verseIdentifier = obj as VerseIdentifier;
         if (verseIdentifier == null)
         {
            return false;
         }
 
         return
            (BookName == verseIdentifier.BookName) &&
            (ChapterNumber == verseIdentifier.ChapterNumber) &&
            (VerseNumber == verseIdentifier.VerseNumber);
      }

      public bool Equals(VerseIdentifier verseIdentifier)
      {
         if ((object)verseIdentifier == null)
         {
            return false;
         }

         // Return true if the fields match:
         return
            (BookName == verseIdentifier.BookName) &&
            (ChapterNumber == verseIdentifier.ChapterNumber) &&
            (VerseNumber == verseIdentifier.VerseNumber);
      }

      public override int GetHashCode()
      {
         return BookName.GetHashCode() ^ ChapterNumber ^ VerseNumber;
      }

      public static bool operator ==(VerseIdentifier a, VerseIdentifier b)
      {
         if (ReferenceEquals(a, b))
         {
            return true;
         }

         if (((object)a == null) || ((object)b == null))
         {
            return false;
         }
      
         return
            a.BookName == b.BookName &&
            a.ChapterNumber == b.ChapterNumber &&
            a.VerseNumber == b.VerseNumber;
      }
      
      public static bool operator !=(VerseIdentifier a, VerseIdentifier b)
      {
         return !(a == b);
      }
   }
}

