using System;

namespace SharpSword
{
   interface IVersificationInternal : IVersification
   {
      /// <summary>
      /// Gets the OSIS book identifier from the zero based book number.
      /// </summary>
      /// <returns>OSIS book identifier</returns>
      /// <param name='bookNumber'>Zero based book number</param>
      string GetOsisIdFromBookNumber (int bookNumber);

      /// <summary>
      /// Gets the absolute chapter number of the first chapter in the book identified
      /// by the OSIS book identifier.
      /// </summary>
      /// <returns>The absolute chapter number</returns>
      /// <param name='osisId'>OSIS book identifier</param>
      int GetAbsoluteChapterNumberFromOsisId (string osisId);

      int GetAbsoluteChapterNumber (string bookName);

      string GetBookName (int absoluteChapterNumber);

      string GetBookNameFromBookNumber (int bookNumber);

      short[][] GetVersesInChapters ();

      /// <summary>
      /// Gets the OSIS book identifier for the given book.
      /// </summary>
      /// <returns>
      /// The OSIS book identifier.
      /// </returns>
      /// <param name='bookName'>Full book name.</param>
      string GetOsisId (string bookName);

      /// <summary>
      /// Gets the name of the book.
      /// </summary>
      /// <returns>
      /// The book name.
      /// </returns>
      /// <param name='osisId'>
      /// Osis identifier.
      /// </param>
      string GetBookName (string osisId);

      bool DoesOsisIdExist (string osisId);

      int GetBookNumberFromBookName(string bookName);
   }
}

