// SharpSword C# library for accessing sword modules
// Copyright (C) 2013  Daniel Hughes
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;

namespace SharpSword
{
   /// <summary>
   /// Versification manager.
   /// </summary>
   class VersificationManager
      : IVersificationManager
   {
      private Dictionary<string, IVersificationInternal> _versifications;
    
      private VersificationManager ()
      {
         _versifications = new Dictionary<string, IVersificationInternal> ();
      }
    
      private static IVersificationManager _instance;

      internal static void InjectInstance(IVersificationManager instance)
      {
         _instance = instance;
      }

      /// <summary>
      /// Gets the VersificationManager instance.
      /// This is a singleton
      /// </summary>
      internal static IVersificationManager Instance
      {
         get
         {
            if (_instance == null)
            {
               _instance = new VersificationManager ();
            }
            return _instance;
         }
      }

      public IVersificationInternal GetVersification(string name)
      {
         IVersificationInternal versification;
         if (!_versifications.TryGetValue (name, out versification))
         {
            return InitializeVersification (name);
         }
         return versification;
      }
    
      private IVersificationInternal InitializeVersification (string name)
      {
         if (name == "kjv")
         {
            var versification = new KjvVersification ();
            _versifications.Add (name, versification);
            return versification;
         }
         string exceptionMessage = 
            string.Format ("Versification type {0} is not supported", name);
         throw new NotImplementedException (exceptionMessage);
      }
   }
}

